import { Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges } from '@angular/core';
import { fadeInAnimation } from '../../../kernel/animations/fade.animation';

import { NewMessageStatus } from './newMessageStatus';

/**
 * Добавление нового сообщения
 */
@Component({
    selector: 'glx-chat-new-message',
    styleUrls: ['./chatNewMessage.scss'],
    templateUrl: './chatNewMessage.html',
    animations: [fadeInAnimation]
})

export class ChatNewMessage implements OnInit, OnChanges {
    /**
     * Статусы отправки сообщений
     * @type {NewMessageStatus}
     */
    NewMessageStatus = NewMessageStatus;
    /**
     * Статус отправки сообщения
     * @type {NewMessageStatus}
     */
    @Input() status: NewMessageStatus;
    /**
     * Текст ошибки отправки сообщения
     * @type {string}
     */
    @Input() error: string;
    /**
     * URL на который нужно отправлять файлы. Подробнее см. fileInput
     * @type {string}
     */
    @Input() uploadFilesUrl: string = '';
    /**
     * Отправлять файлы с JWT токеном или без него
     * @type {boolean}
     */
    @Input() uploadFilesWithAuth: boolean = false;
    /**
     * Новое сообщение
     * @type {EventEmitter<any>}
     */
    @Output() newMessage: EventEmitter<any> = new EventEmitter();
    /**
     * Содержимое нового сообщения
     */
    model: any;
    /**
     * Файлы, приложенные к сообщению
     */
    _files: any[];
    editor: any;

    /**
     * Конструктор
     */
    constructor() {
    }

    /**
     * Инициализация
     */
    ngOnInit() {
        this.initModel();
    }

    ngOnChanges(changes: SimpleChanges) {
        if (changes.status && !changes.status.isFirstChange() && changes.status.currentValue === NewMessageStatus.newMessage) {
            this.initModel();
        }
    }

    initEditor(e) {
        console.log(e);
        this.editor = e.editor;
    }

    /**
     * Добавляет новое сообщение
     */
    add() {
        if (this.status === NewMessageStatus.newMessage) {
            this.editor.disable();
            this.status = NewMessageStatus.sending;
            this.newMessage.emit(this.model);

            /**
             * Выдаем пользователю ошибку, если через 10 секунд после отправки сообщение все еще отправляется
             */
            setTimeout(() => {
                if (this.status === NewMessageStatus.sending) {
                    this.status = NewMessageStatus.error;
                    this.error = 'Не удается отправить сообщение. Попробуйте повторить операцию через некоторое время.';
                    setTimeout(() => {
                        this.status = NewMessageStatus.newMessage;
                    }, 2000);
                }
            }, 10000);
        }
    }

    /**
     * Вызывается при изменении файлов
     * @param event
     */
    changeFiles(event) {
        this._files = event.files;
        this.formatFiles();
    }

    /**
     * Получить иконку для кнопки отправки сообщения по статусу отправки
     * @returns {string} класс иконки Font Awesome
     */
    getStatusIcon() {
        switch (this.status) {
            case NewMessageStatus.newMessage:
                return 'fa-paper-plane-o';
            case NewMessageStatus.sending:
                return 'fa-spinner fa-spin';
            case NewMessageStatus.OK:
                return 'fa-check text-success';
            case NewMessageStatus.error:
                return 'fa-times text-danger';
            default:
                return 'fa-paper-plane-o';
        }
    }

    /**
     * Преобразовывает приложенные файлы к формату для отправки
     */
    private formatFiles() {
        this.model.attachments = [];
        for (let i = 0; i < this._files.length; i++) {
            this.model.attachments.push({
                path: this._files[i].path,
                name: this._files[i].name
            });
        }
    }

    /**
     * Инициализация нового сообщения
     */
    private initModel() {
        this.model = {
            message: null,
            attachments: null
        };
        this.status = NewMessageStatus.newMessage;
        if (this.editor) {
            this.editor.enable();
        }
    }

}
